;DataListGadget 
;
;Befehlsbeschreibung:
; ________________________________________________________________________________________________________________
;| -DataListGadget_Create(GadgetNr.l, X.l, Y.l, Breite.l, Hoehe.l, ItemHoehe.l = 20, Hintergrundfarbe.l = $fcfcfc)|
;|Erstellt ein DataListGadget an den angegebenen Koordinaten mit der den angegebenen Dimensionen.                 |
;|Die `ItemHoehe` bestimmt die Schriftgrsse und die Hhe der Items.                                              |
;|________________________________________________________________________________________________________________|
;| -DataListGadget_AddItem(GadgetNr.l, Pos.i, Beschreibung$, Text$, ItemArt)                                      |
;|Fgt einen neuen Eintrag mit der angegebenen Beschreibung und Text hinzu.                                       |
;|Wenn bei `Pos` -1 angegeben wird, dann wird der Eintrag am Ende der Liste eingefgt.                            |
;|________________________________________________________________________________________________________________|
;| -DataListGadget_AddComboItem(GadgetNr.l, Item.i, Text$)                                                        |
;|Fgt dem angegebenen Eintrag ein Combobox-Item hinzu                                                            |
;|________________________________________________________________________________________________________________|
;| -DataListGadget_GetItemText(GadgetNr.l, Item.i)                                                                |
;|Ermittelt den Text des angegebenen Items.                                                                       |
;|________________________________________________________________________________________________________________|
;| -DataListGadget_SetItemText(GadgetNr.l, Item.i, Text$)                                                         |
;|ndert den Text des angegebenen Items.                                                                          |
;|________________________________________________________________________________________________________________|
;| -DataListGadget_Refresh(GadgetNr.l, WindowEvent.i)                                                             |
;|Dieser Befehl berechnet und zeichnet das Gadget und sollte in der Hauptschleife aufgerufen werden.              |
;|`WindowEvent` wird fr die Tastendrcke verwendet.                                                              |
;|________________________________________________________________________________________________________________|

;ItemArten
#Item_Text = 1
#Item_Combo = 2

Structure _Items
  Beschreibung$
  Text$
  CursorPos.i
  ItemArt.i
  AktuellesComboItem.i
  List ComboItems.s()
EndStructure

Structure _DataListGadget
  GadgetNr.l
  X.l
  Y.l
  Breite.l
  GesBreite.l
  Hoehe.l
  ItemHoehe.l
  Hintergrundfarbe.l
  Font.i
  AktuellesItem.l
  Focus.i
  ScrollBar.l
  List Items._Items()
EndStructure

Procedure.s RemoveStringSign(String$, Pos.i)
  Left$ = Left(String$, Pos)
  Right$ = Right(String$, Len(String$)-(Pos+1))
  ProcedureReturn Left$+Right$ 
EndProcedure


Procedure DataListGadget_Create(GadgetNr.l, X.l, Y.l, Breite.l, Hoehe.l, ItemHoehe.l = 20, Hintergrundfarbe.l = $fcfcfc)
  *DataList._DataListGadget = AllocateMemory(SizeOf(_DataListGadget))
  If *DataList
    InitializeStructure(*DataList, _DataListGadget)
    *DataList\X = X
    *DataList\Y = Y
    *DataList\Breite = Breite-20
    *DataList\GesBreite = Breite
    *DataList\Hoehe = Hoehe
    *DataList\ItemHoehe = ItemHoehe
    *DataList\Hintergrundfarbe = Hintergrundfarbe
    *DataList\Font = LoadFont(#PB_Any, "Arial", ItemHoehe*0.5, #PB_Font_HighQuality)
    *DataList\ScrollBar = ScrollBarGadget(#PB_Any, Breite-20, Y, 20, Hoehe, 0, 210, Hoehe, #PB_ScrollBar_Vertical)
    If GadgetNr = #PB_Any
      *DataList\GadgetNr = CanvasGadget(GadgetNr, X, Y, Breite-20, Hoehe, #PB_Canvas_Keyboard)
      SetGadgetData(*DataList\GadgetNr, *DataList)
      ProcedureReturn *DataList\GadgetNr
    Else
      *DataList\GadgetNr = GadgetNr
      CanvasGadget(GadgetNr, X, Y, Breite-20, Hoehe, #PB_Canvas_Keyboard)
      SetGadgetData(GadgetNr, *DataList)
      ProcedureReturn #True
    EndIf
  EndIf
EndProcedure

Procedure DataListGadget_AddItem(GadgetNr.l, Pos.i, Beschreibung$, Text$, ItemArt.i = #Item_Text)
  *DataList._DataListGadget = GetGadgetData(GadgetNr)
  If *DataList
    With *DataList
      If Pos = -1
        LastElement(\Items())  
      Else
        If Pos < ListSize(\Items()) And Pos >=0
          SelectElement(\Items(), Pos)
        EndIf  
      EndIf
      AddElement(\Items())
      \Items()\Beschreibung$ = Beschreibung$
      \Items()\Text$ = Text$
      \Items()\ItemArt = ItemArt
      SetGadgetAttribute(\ScrollBar, #PB_ScrollBar_Maximum, \ItemHoehe * ListSize(\Items())+10) 
      SetGadgetAttribute(\ScrollBar, #PB_ScrollBar_PageLength, \Hoehe)
      If \Hoehe > \ItemHoehe * ListSize(\Items())+10 
        \Breite = \GesBreite
        ResizeGadget(\ScrollBar, \Breite, #PB_Ignore, 0, #PB_Ignore)
      Else 
        \Breite = \GesBreite-20
        ResizeGadget(\ScrollBar, \Breite+5, #PB_Ignore, 20, #PB_Ignore)
      EndIf
      ResizeGadget(\GadgetNr, #PB_Ignore, #PB_Ignore, \Breite, #PB_Ignore)
      SetGadgetData(\GadgetNr, *DataList)
    EndWith
  EndIf
EndProcedure

Procedure DataListGadget_AddComboItem(GadgetNr.l, Item.i, Text$)
  *DataList._DataListGadget = GetGadgetData(GadgetNr)
  If *DataList
    SelectElement(*DataList\Items(), Item)
    LastElement(*DataList\Items()\ComboItems())
    AddElement(*DataList\Items()\ComboItems())
    *DataList\Items()\ComboItems() = Text$
    SetGadgetData(GadgetNr, *DataList)
  EndIf
EndProcedure

Procedure.s DataListGadget_GetItemText(GadgetNr.l, Item.i)
  *DataList._DataListGadget = GetGadgetData(GadgetNr)
  If *DataList
    If Item < ListSize(*DataList\Items()) And Item >=0
      SelectElement(*DataList\Items(), Item)
      ProcedureReturn *DataList\Items()\Text$
    EndIf
  EndIf
EndProcedure

Procedure DataListGadget_SetItemText(GadgetNr.l, Item.i, Text$)
  *DataList._DataListGadget = GetGadgetData(GadgetNr)
  If *DataList
    If Item < ListSize(*DataList\Items()) And Item >=0
      SelectElement(*DataList\Items(), Item)
      *DataList\Items()\Text$ = Text$
      SetGadgetData(*DataList\GadgetNr, *DataList)
    EndIf
  EndIf
EndProcedure

Procedure DataListGadget_GetItemState(GadgetNr, Item.i)
  *DataList._DataListGadget = GetGadgetData(GadgetNr)
  If *DataList
    If Item < ListSize(*DataList\Items()) And Item >=0
      SelectElement(*DataList\Items(), Item)
      ProcedureReturn *DataList\Items()\AktuellesComboItem
    EndIf
  EndIf
EndProcedure

Procedure DataListGadget_Refresh(GadgetNr.l, WindowEvent.i)
  Static TextBreite.i, Popupmenu.i, ScrollPos
  *DataList._DataListGadget = GetGadgetData(GadgetNr)
  If *DataList 
    With *DataList
      SelectElement(\Items(), \AktuellesItem)
      If WindowEvent = #PB_Event_Gadget And EventGadget() = \GadgetNr 
        Select EventType() 
          Case #PB_EventType_Input
            If \Items()\ItemArt = #Item_Text
              \Items()\Text$ = InsertString(\Items()\Text$, Chr(GetGadgetAttribute(\GadgetNr, #PB_Canvas_Input)), \Items()\CursorPos+1)
              \Items()\CursorPos+1
            EndIf
          Case #PB_EventType_Focus
            \Focus = 1
          Case #PB_EventType_LostFocus
            \Focus = 0
          Case #PB_EventType_LeftClick
            If (GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseY)-5)/\ItemHoehe < ListSize(\Items())
              If Popupmenu = 0 : \AktuellesItem = (GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseY)-5+GetGadgetState(\ScrollBar))/\ItemHoehe : EndIf
              If \Items()\ItemArt = #Item_Text
                Popupmenu = 0
                X = GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseX)-\Breite/2
                If \Items()\Text$ 
                  \Items()\CursorPos = Round(Len(\Items()\Text$)/TextBreite*X, #PB_Round_Nearest) 
                  If \Items()\CursorPos > Len(\Items()\Text$) : \Items()\CursorPos = Len(\Items()\Text$) : EndIf 
                EndIf
              ElseIf \Items()\ItemArt = #Item_Combo
                If GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseX) > \Breite-\ItemHoehe 
                  If Popupmenu = 0
                    Popupmenu = 1 : Else : Popupmenu = 0 
                  EndIf   
                Else 
                  If Popupmenu And GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseY) > (\AktuellesItem+1)*\ItemHoehe-ScrollPos And GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseY) < (\AktuellesItem+1+ListSize(\Items()\ComboItems()))*\ItemHoehe-ScrollPos
                    \Items()\AktuellesComboItem = Round((GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseY)-((\AktuellesItem+1)*\ItemHoehe)+ScrollPos) / \ItemHoehe, #PB_Round_Down)
                    SelectElement(\Items()\ComboItems(), \Items()\AktuellesComboItem)
                    \Items()\Text$ = \Items()\ComboItems()
                    Popupmenu = 0
                  Else
                    Popupmenu = 0
                  EndIf
                EndIf
              EndIf
            EndIf
          Case #PB_EventType_MouseMove
            If GetGadgetAttribute(\GadgetNr, #PB_Canvas_MouseX) > \Breite/2 
              SetGadgetAttribute(\GadgetNr, #PB_Canvas_Cursor, #PB_Cursor_IBeam)
            Else
              SetGadgetAttribute(\GadgetNr, #PB_Canvas_Cursor, #PB_Cursor_Default)
            EndIf
        EndSelect
      EndIf
      If WindowEvent = #WM_KEYDOWN And \Focus 
        Select EventwParam()
          Case #VK_BACK  : If \Items()\CursorPos > 0 And \Items()\ItemArt = #Item_Text : \Items()\Text$ = RemoveStringSign(\Items()\Text$, \Items()\CursorPos-1) : \Items()\CursorPos -1 : EndIf
          Case #VK_LEFT  : If \Items()\CursorPos > 0 : \Items()\CursorPos - 1 : EndIf
          Case #VK_RIGHT : If \Items()\CursorPos < Len(\Items()\Text$) : \Items()\CursorPos + 1 : EndIf
          Case #VK_UP    : If \AktuellesItem > 0 : \AktuellesItem-1 : EndIf
          Case #VK_DOWN  : If \AktuellesItem < ListSize(\Items())-1 : \AktuellesItem+1 : EndIf
        EndSelect
      EndIf
      
      If StartDrawing(CanvasOutput(\GadgetNr))
        Box(0, 0, \Breite, \Hoehe, \Hintergrundfarbe)
        DrawingMode(#PB_2DDrawing_Outlined|#PB_2DDrawing_Transparent)
        DrawingFont(FontID(\Font))
        ScrollPos = GetGadgetState(\ScrollBar)
        ForEach \Items()
          Y = ListIndex(\Items())*\ItemHoehe 
          Box(0, Y-ScrollPos, \Breite, \ItemHoehe+1, $cccccc)
          Line(\Breite/2, Y-ScrollPos, 1, \ItemHoehe, $cccccc)
          If ListIndex(\Items()) = \AktuellesItem
            DrawingMode(#PB_2DDrawing_Default)
            Box(1, 1+\AktuellesItem*\ItemHoehe-ScrollPos, \Breite/2-1, \ItemHoehe-1, $FF9933)
            Line(\Breite/2+2+TextWidth(Left(\Items()\Text$, \Items()\CursorPos)), 1+\ItemHoehe*\AktuellesItem-ScrollPos, 1, \ItemHoehe, $202020)
            TextBreite = TextWidth(\Items()\Text$)
            DrawingMode(#PB_2DDrawing_Outlined|#PB_2DDrawing_Transparent)
            If \Items()\ItemArt = #Item_Combo
              Box(\Breite-\ItemHoehe, 1+\AktuellesItem*\ItemHoehe-ScrollPos, \ItemHoehe-1, \ItemHoehe-1, $202020)
              FillArea(\Breite-\ItemHoehe+1, 2+\AktuellesItem*\ItemHoehe-ScrollPos, $202020, $dedede)
              DrawRotatedText(\Breite-\ItemHoehe/1.1, \AktuellesItem*\ItemHoehe-ScrollPos+\ItemHoehe/1.4, Chr(17), 90, $202020)
            EndIf
            farbe = \Hintergrundfarbe
          Else
            farbe = $101010
          EndIf 
          DrawText(2, 1+Y+\ItemHoehe/2-TextHeight("1")/2-ScrollPos, \Items()\Beschreibung$, farbe, \Hintergrundfarbe)
          DrawText(\Breite/2+2, 1+Y+\ItemHoehe/2-TextHeight("1")/2-ScrollPos, \Items()\Text$, $101010, \Hintergrundfarbe)    
        Next
        If Popupmenu
          SelectElement(\Items(), \AktuellesItem)
          DrawingMode(#PB_2DDrawing_Default|#PB_2DDrawing_Transparent)
          ForEach \Items()\ComboItems()
            Box(\Breite/2, (\AktuellesItem+1)*\ItemHoehe+ListIndex(\Items()\ComboItems())*\ItemHoehe-ScrollPos, \Breite/2, \ItemHoehe, $efefef)
            DrawText(\Breite/2+1, (\AktuellesItem+1)*\ItemHoehe+ListIndex(\Items()\ComboItems())*\ItemHoehe+\ItemHoehe/2-TextHeight("?")/2-ScrollPos, \Items()\ComboItems(), $202020)
          Next
        EndIf
        StopDrawing()
      EndIf
      SetGadgetData(\GadgetNr, *DataList)
    EndWith
  EndIf
EndProcedure

; IDE Options = PureBasic 4.60 (Windows - x86)
; CursorPosition = 263
; FirstLine = 219
; Folding = --
; EnableXP